# Copyright (C) 2013-2017 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from time import asctime, gmtime
import gdb  # silence pyflakes


class TimePrinter:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        secs = int(self.val)
        return "%s (%d)" % (asctime(gmtime(secs)), secs)


def time_sniffer(val):
    if hasattr(val.type, 'name') and val.type.name == "time_t":
        return TimePrinter(val)
    return None


gdb.pretty_printers.append(time_sniffer)



# Copyright (C) 2015-2017 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This file is part of the GDB testsuite.  It tests python unwinders.

import re
import gdb.types
from gdb.unwinder import Unwinder, register_unwinder

class TestGlobalUnwinder(Unwinder):
    def __init__(self):
        super(TestGlobalUnwinder, self).__init__("global_unwinder")

    def __call__(self, unwinder_info):
        print("%s called" % self.name)
        return None

class TestProgspaceUnwinder(Unwinder):
    def __init__(self, name):
        super(TestProgspaceUnwinder, self).__init__("%s_ps_unwinder" % name)

    def __call__(self, unwinder_info):
        print("%s called" % self.name)
        return None

class TestObjfileUnwinder(Unwinder):
    def __init__(self, name):
        super(TestObjfileUnwinder, self).__init__("%s_obj_unwinder" % name)

    def __call__(self, unwinder_info):
        print("%s called" % self.name)
        return None



gdb.unwinder.register_unwinder(None, TestGlobalUnwinder())
saw_runtime_error = False
try:
    gdb.unwinder.register_unwinder(None, TestGlobalUnwinder(), replace=False)
except RuntimeError:
    saw_runtime_error = True
if not saw_runtime_error:
    raise RuntimeError("Missing runtime error from register_unwinder.")
gdb.unwinder.register_unwinder(None, TestGlobalUnwinder(), replace=True)
gdb.unwinder.register_unwinder(gdb.current_progspace(),
                               TestProgspaceUnwinder("py_unwind_maint"))
print("Python script imported")



#  This testcase is part of GDB, the GNU debugger.

#  Copyright 2008-2017 Free Software Foundation, Inc.

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

print ('y%ss' % 'e')



# Copyright (C) 2013-2017 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import gdb

class pp_s (object):
    def __init__(self, val):
        self.val = val

    def to_string(self):
        m = self.val["m"]
        return "m=<" + str(self.val["m"]) + ">"

class pp_ss (object):
    def __init__(self, val):
        self.val = val

    def to_string(self):
        return "super struct"

    def children (self):
        yield 'a', self.val['a']
        yield 'b', self.val['b']


def lookup_function (val):
    "Look-up and return a pretty-printer that can print val."

    # Get the type.
    type = val.type

    # If it points to a reference, get the reference.
    if type.code == gdb.TYPE_CODE_REF:
        type = type.target ()

    # Get the unqualified type, stripped of typedefs.
    type = type.unqualified ().strip_typedefs ()

    # Get the type name.    
    typename = type.tag
    if typename == None:
        return None

    # Iterate over local dictionary of types to determine
    # if a printer is registered for that type.  Return an
    # instantiation of the printer if found.
    for function in pretty_printers_dict:
        if function.match (typename):
            return pretty_printers_dict[function] (val)
        
    # Cannot find a pretty printer.  Return None.
    return None


def register_pretty_printers ():
    pretty_printers_dict[re.compile ('^s$')] = pp_s
    pretty_printers_dict[re.compile ('^ss$')] = pp_ss

pretty_printers_dict = {}

register_pretty_printers ()
gdb.pretty_printers.append (lookup_function)



# Extended prompt.
# Copyright (C) 2011-2017 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""GDB command for working with extended prompts."""

import gdb
import gdb.prompt

class _ExtendedPrompt(gdb.Parameter):

    """Set the extended prompt.

Usage: set extended-prompt VALUE

Substitutions are applied to VALUE to compute the real prompt.

The currently defined substitutions are:

"""
    # Add the prompt library's dynamically generated help to the
    # __doc__ string.
    __doc__ = __doc__ + gdb.prompt.prompt_help()

    set_doc = "Set the extended prompt."
    show_doc = "Show the extended prompt."

    def __init__(self):
        super(_ExtendedPrompt, self).__init__("extended-prompt",
                                              gdb.COMMAND_SUPPORT,
                                              gdb.PARAM_STRING_NOESCAPE)
        self.value = ''
        self.hook_set = False

    def get_show_string (self, pvalue):
        if self.value is not '':
           return "The extended prompt is: " + self.value
        else:
           return "The extended prompt is not set."

    def get_set_string (self):
        if self.hook_set == False:
           gdb.prompt_hook = self.before_prompt_hook
           self.hook_set = True
        return ""

    def before_prompt_hook(self, current):
        if self.value is not '':
            return gdb.prompt.substitute_prompt(self.value)
        else:
            return None

_ExtendedPrompt()
